@@ -11,7 +11,8 @@ from suite_py.lib.handler.drone_handler import DroneHandler
 from suite_py.lib.handler.git_handler import GitHandler
 from suite_py.lib.handler.github_handler import GithubHandler
 from suite_py.lib.handler.youtrack_handler import YoutrackHandler
-from suite_py.lib.qainit import qainit_shutdown
+from suite_py.lib.handler.qainit_handler import QainitHandler
+
 from suite_py.lib.symbol import CHECKMARK, CROSSMARK
 
 
@@ -24,6 +25,7 @@ class MergePR:
         self._git = GitHandler(project, config)
         self._github = GithubHandler(tokens)
         self._drone = DroneHandler(config, tokens, repo=project)
+        self._qainit = QainitHandler(project, config, tokens)
 
     def run(self):
         self._stop_if_master_locked()
@@ -80,8 +82,34 @@ class MergePR:
             )
             logger.info("Card updated")
 
-            logger.info("Turning off the QA, if it exists")
-            qainit_shutdown(youtrack_id, self._config)
+            with Halo(
+                text="Searching for related QA environments...",
+                spinner="dots",
+                color="magenta",
+            ):
+                response = self._qainit.list(
+                    {
+                        "user": "all",
+                        "card": youtrack_id,
+                        "status": ["created", "updated", "failed", "frozen"],
+                    }
+                ).json()
+
+            for qa in response["list"]:
+                logger.info(f"Found {qa['name']}")
+
+            if (
+                self._config.user["delete_qa_after_merge"]
+                and len(response["list"]) > 0
+                and prompt_utils.ask_confirm(
+                    "Do you want to delete the QA env(s) I found?",
+                    default=True,
+                )
+            ):
+                for qa in response["list"]:
+                    logger.info(f"Deleting {qa['name']} ")
+                    self._qainit.delete(qa["hash"])
+
         else:
             logger.warning(
                 "There is no YouTrack issue in the branch name or the selected issue does not exist."
