@@ -3,6 +3,7 @@
 import sys
 import json
 import copy
+import re
 
 from rich.table import Table
 from rich.console import Console
@@ -11,6 +12,7 @@ from suite_py.lib.handler import prompt_utils
 from suite_py.lib.handler.qainit_handler import QainitHandler
 from suite_py.lib.handler import git_handler as git
 from suite_py.lib.handler.git_handler import GitHandler
+from suite_py.lib.handler.youtrack_handler import YoutrackHandler
 
 
 class QA:
@@ -22,6 +24,7 @@ class QA:
         self._tokens = tokens
         self._git = GitHandler(project, config)
         self._qainit = QainitHandler(project, config, tokens)
+        self._youtrack = YoutrackHandler(config, tokens)
 
     def run(self):
         if self._action == "list":
@@ -44,11 +47,11 @@ class QA:
             self._update_quota()
 
     def _check(self):
-        r = self._qainit.execute("GET", "/api/v1/user")
         logger.info(
             "Checking configuration. If there is an issue, check ~/.suite_py/config.yml file and execute: suite-py login"
         )
-        logger.debug(json.dumps(r.json(), indent=2))
+
+        self._qainit.user_info()
 
     def _list(self):
         # init empty table with column (useful for reset)
@@ -63,29 +66,13 @@ class QA:
         table = copy.deepcopy(empty_table)
         console = Console()
 
-        # add filtering
-        filters = []
-        status_values = '"created","creating","updated","updating","failed","frozen"'
-        if len(self._flags["status"]) > 0:
-            status_values = ",".join(
-                [f'"{status}"' for status in self._flags["status"]]
-            )
-        filters.append(f"status=[{status_values}]")
-
-        if self._flags["user"] is not None:
-            filters.append(f"user={self._flags['user']}")
-
-        if self._flags["card"] is not None:
-            filters.append(f"card={self._flags['card']}")
-
-        filters_string = "&".join(filters)
-
         # execute query with pagination and filtering
         page_number = 1
         while True:
-            r = self._qainit.execute(
-                "GET",
-                f'/api/v1/qa?{filters_string}&page_size={self._config.qainit["table_size"]}&page={page_number}',
+            r = self._qainit.list(
+                self._flags,
+                page=page_number,
+                page_size=self._config.qainit["table_size"],
             )
             response = r.json()
             qa_list = response["list"]
@@ -120,15 +107,12 @@ class QA:
             table = copy.deepcopy(empty_table)
 
     def _describe(self):
-        qa_hash = self._flags["qa_hash"]
+        qa_hash = self._flags["hash"]
         jsonify = self._flags["json"]
 
-        r = self._qainit.execute(
-            "GET",
-            f"/api/v1/qa/{qa_hash}",
-        )
+        r = self._qainit.describe(qa_hash)
         if jsonify:
-            print(json.dumps(r.json(), sort_keys=True, indent=2))
+            print(json.dumps(r, sort_keys=True, indent=2))
         else:
             table = Table()
             table.add_column("Microservice", style="purple", no_wrap=True)
@@ -143,9 +127,7 @@ class QA:
             console = Console()
 
             try:
-                resources_list = sorted(
-                    r.json()["list"]["resources"], key=lambda k: k["name"]
-                )
+                resources_list = sorted(r["list"]["resources"], key=lambda k: k["name"])
                 for resource in resources_list:
                     if (
                         resource["type"] == "microservice"
@@ -177,39 +159,21 @@ class QA:
                 logger.error("Wrong hash")
 
     def _delete(self):
-        qa_hash = self._flags
-
-        r = self._qainit.execute(
-            "DELETE",
-            f"/api/v1/qa/{qa_hash}",
-        )
-        logger.info("QA deletion initiated")
-        logger.debug(json.dumps(r.json(), indent=2))
+        self._qainit.delete(self._flags["hash"])
 
     def _freeze(self):
-        qa_hash = self._flags
-
-        body = {"operation": "freeze"}
-        logger.debug(json.dumps(body))
-        r = self._qainit.execute("PUT", f"/api/v1/qa/{qa_hash}", body=json.dumps(body))
-        logger.info("QA freezing initiated")
-        logger.debug(json.dumps(r.json(), indent=2))
+        self._qainit.freeze(self._flags["hash"])
 
     def _unfreeze(self):
-        qa_hash = self._flags
-
-        body = {"operation": "unfreeze"}
-        logger.debug(json.dumps(body))
-        r = self._qainit.execute("PUT", f"/api/v1/qa/{qa_hash}", body=json.dumps(body))
-        logger.info("QA unfreezing initiated")
-        logger.debug(json.dumps(r.json(), indent=2))
+        self._qainit.unfreeze(self._flags["hash"])
 
     def _create(self):
-        r = self._qainit.execute("GET", "/api/v1/user")
-        r = r.json()
-        if not r["quota"]["remaining"] > 0:
+        user = self._qainit.user_info()
+
+        if not user["quota"]["remaining"] > 0:
             logger.error("There's no remaining quota for you.")
             sys.exit("-1")
+
         if "staging" in self._qainit.url:
             qa_default_name = (
                 f"staging_{git.get_username()}_{self._git.current_branch_name()}"
@@ -221,49 +185,29 @@ class QA:
             "Choose the QA name: ", default_text=qa_default_name
         )
 
+        card_match = re.match(r"[^\/]*_(?P<name>[A-Z]+-\d+)\/", qa_name)
+        default_card_name = (
+            card_match["name"] if card_match else self._config.user["default_slug"]
+        )
+
         qa_card = prompt_utils.ask_questions_input(
-            "Youtrack issue ID: ", default_text="PRIMA-xxxx"
+            "Youtrack issue ID: ", default_text=default_card_name
         )
 
-        srv_list = self._qainit.create_services_body(prj_list=self._flags)
+        if qa_card != "":
+            try:
+                self._youtrack.get_issue(qa_card)
+            except Exception:
+                logger.error("invalid Youtrack issue ID")
+                sys.exit(-1)
 
-        body = {"name": qa_name, "card": qa_card, "services": srv_list}
-        logger.debug(json.dumps(body))
-        r = self._qainit.execute(
-            "POST",
-            "/api/v1/qa",
-            body=json.dumps(body),
-        )
-        logger.info(f"QA creation initiated. Your namespace hash: {r.json()['hash']}")
-        logger.debug(json.dumps(r.json(), indent=2))
+        self._qainit.create(qa_name, qa_card, self._flags["services"])
 
     def _update(self):
-        qa_hash = self._flags[0]
-        prj_list = self._flags[1:]
-
-        srv_list = self._qainit.create_services_body(prj_list)
-
-        body = {"services": srv_list}
-        logger.debug(json.dumps(body))
-        r = self._qainit.execute(
-            "PUT",
-            f"/api/v1/qa/{qa_hash}",
-            body=json.dumps(body),
-        )
-        logger.info("QA update initiated")
-        logger.debug(json.dumps(r.json(), indent=2))
+        self._qainit.update(self._flags["hash"], self._flags["services"])
 
     def _update_quota(self):
         username = prompt_utils.ask_questions_input("Insert GitHub username: ")
         quota = prompt_utils.ask_questions_input("Insert new quota value: ")
 
-        body = {"github_username": f"{username}", "quota": f"{quota}"}
-        logger.debug(json.dumps(body))
-        r = self._qainit.execute(
-            "POST",
-            "/api/v1/user/quota",
-            body=json.dumps(body),
-        )
-
-        logger.info("Quota updated.")
-        logger.debug(json.dumps(r.json(), indent=2))
+        self._qainit.update_user_quota(username, quota)
